var fs = require('fs');
var path = require('path');
var exec = require('child_process').exec;
var optimist = require('optimist');
var cli = optimist.usage('Create a report about unused icons in a project.' + '\n' + 'Usage: $0 path/to/your/project/');
var projectDir = cli.argv._[0];
if (!projectDir) {
    cli.showHelp();
    process.exit(0);
}
var stats = fs.lstatSync(projectDir);
if (!stats.isDirectory()) {
    console.log('\x1B[31mPlease provide a valid folder.\x1B[0m\n');
    cli.showHelp();
    process.exit(0);
}
function getIcons() {
    var icons = fs.readdirSync('./images/');
    icons = icons.map(function (icon) {
        return path.basename(icon, '.svg');
    });
    return icons;
}
function getUsedIcons(grep, callback) {
    exec(grep, function (err, stdout, stderr) {
        var usedIcons = stdout.trim().split('\n').map(function (icon) {
            return icon.split(/["']/)[1];
        }).filter(function (icon) {
            return icon && icon.match(/^[a-z0-9-]+$/);
        });
        callback(usedIcons);
        if (err) {
            console.log(err);
        }
    });
}
function getUnusedIcons() {
    var usedIcons = [];
    var length = 2;
    var cb = function (icons) {
        usedIcons = usedIcons.concat(icons);
        length--;
        if (length > 0) {
            return;
        }
        var allIcons = getIcons();
        var unusedIcons = [];
        allIcons.forEach(function (icon) {
            if (usedIcons.indexOf(icon) === -1) {
                unusedIcons.push(icon);
            }
        });
        outputUnusedIcons(allIcons, unusedIcons);
    };
    getUsedIcons('grep -rhoP \'data-icon="[^"]+"\' ' + projectDir, cb);
    getUsedIcons('grep -rhoP \'.dataset.icon ?= ?.+\' ' + projectDir, cb);
}
function outputUnusedIcons(icons, unusedIcons) {
    console.log('\x1B[36mList of unused icons in %s (%d/%d):\x1B[0m', projectDir, unusedIcons.length, icons.length);
    unusedIcons.forEach(function (icon) {
        console.log('* ' + icon);
    });
}
getUnusedIcons();