define([
    'require',
    'l10n!',
    'cards',
    './oauth2/fetch',
    './base_card',
    'template!./settings_account_credentials.html'
], function (require) {
    var mozL10n = require('l10n!'), cards = require('cards'), oauthFetch = require('./oauth2/fetch');
    return [
        require('./base_card')(require('template!./settings_account_credentials.html')),
        {
            onArgs: function (args) {
                this.account = args.account;
                this.headerLabel.textContent = this.account.name;
                if (this.account.authMechanism !== 'password') {
                    this.querySelector('.tng-account-server-password').classList.add('collapsed');
                    this.saveButton.classList.add('collapsed');
                }
                if (this.account.authMechanism === 'oauth2') {
                    this.reauthButton.classList.remove('collapsed');
                }
                this.usernameNodeInput.value = this.account.username;
            },
            onBack: function () {
                cards.removeCardAndSuccessors(this, 'animate', 1);
            },
            onClickSave: function () {
                var password = this.passwordNodeInput.value;
                if (password) {
                    this.account.modifyAccount({ password: password });
                    this.account.clearProblems();
                } else {
                    alert(mozL10n.get('settings-password-empty'));
                }
                this.onBack();
            },
            onClickReauth: function () {
                var oauth2 = this.account._wireRep.credentials.oauth2;
                oauthFetch(oauth2, { login_hint: this.account.username }).then(function (response) {
                    if (response.status === 'success') {
                        this.account.modifyAccount({ oauthTokens: response.tokens });
                        this.account.clearProblems();
                        this.onBack();
                    }
                }.bind(this));
            },
            die: function () {
            }
        }
    ];
});