function getImageSize(blob, callback, error) {
    'use strict';
    BlobView.get(blob, 0, Math.min(1024, blob.size), function (data) {
        if (data.byteLength <= 8) {
            error('corrupt image file');
            return;
        }
        var magic = data.getBinaryText(0, 8);
        if (magic.substring(0, 4) === 'GIF8') {
            try {
                callback({
                    type: 'gif',
                    width: data.getUint16(6, true),
                    height: data.getUint16(8, true)
                });
            } catch (e) {
                error(e.toString());
            }
        } else if (magic.substring(0, 8) === '\x89PNG\r\n\x1A\n') {
            try {
                callback({
                    type: 'png',
                    width: data.getUint32(16, false),
                    height: data.getUint32(20, false)
                });
            } catch (e) {
                error(e.toString());
            }
        } else if (magic.substring(0, 2) === 'BM' && data.getUint32(2, true) === blob.size) {
            try {
                var width, height;
                if (data.getUint16(14, true) === 12) {
                    width = data.getUint16(18, true);
                    height = data.getUint16(20, true);
                } else {
                    width = data.getUint32(18, true);
                    height = data.getUint32(22, true);
                }
                callback({
                    type: 'bmp',
                    width: width,
                    height: height
                });
            } catch (e) {
                error(e.toString());
            }
        } else if (magic.substring(0, 2) === '\xFF\xD8') {
            parseJPEGMetadata(blob, function (metadata) {
                if (metadata.progressive) {
                    delete metadata.progressive;
                    metadata.type = 'pjpeg';
                } else {
                    metadata.type = 'jpeg';
                }
                callback(metadata);
            }, error);
        } else {
            error('unknown image type');
        }
    });
}